@@ -2,9 +2,9 @@
 ### ====================================================================
 ###  @UNIX-shell-file{
 ###     author          = "Nelson H. F. Beebe",
-###     version         = "0.07",
-###     date            = "17 January 1996",
-###     time            = "05:57:04 MST",
+###     version         = "0.09",
+###     date            = "15 January 1997",
+###     time            = "10:17:08 MST",
 ###     filename        = "biborder.sh",
 ###     address         = "Center for Scientific Computing
 ###                        Department of Mathematics
@@ -13,7 +13,7 @@
 ###                        USA",
 ###     telephone       = "+1 801 581 5254",
 ###     FAX             = "+1 801 581 4148",
-###     checksum        = "07194 831 2848 24194",
+###     checksum        = "20000 849 2920 24749",
 ###     email           = "beebe@math.utah.edu (Internet)",
 ###     codetable       = "ISO/ASCII",
 ###     keywords        = "bibliography, ordering, BibTeX",
@@ -43,6 +43,7 @@
 ###
 ###                        Usage:
 ###                              biborder
+###                                     [-alphabetical]
 ###					[-bimonthly m1/m2/m3/m4/m5/m6 ]
 ###					[-check-missing]
 ###					[-monthly]
@@ -50,6 +51,7 @@
 ###					bibfile(s) >outfile
 ###                        or
 ###                              biborder
+###                                     [-alphabetical]
 ###					[-bimonthly m1/m2/m3/m4/m5/m6 ]
 ###					[-check-missing]
 ###					[-monthly]
@@ -59,6 +61,10 @@
 ###			   Switch names may be abbreviated to the
 ###			   minimal unique prefix.
 ###
+###                        If -alphabetical is specified, the standard
+###                        order of keys is ignored, and strict
+###                        alphabetical order is used instead.
+###
 ###			   If -check-missing is specified, missing
 ###			   expected fields will be supplied, with the
 ###			   field name prefixed with OPT, and the value
@@ -166,6 +172,7 @@
 ########################################################################
 
 # Assign default initial values
+ALPHABETICAL=0
 FILES=
 BIMONTHLY=0
 CHECKMISSING=0
@@ -177,6 +184,9 @@ QUARTERLY=0
 while [ $# -gt -0 ]
 do
 	case $1 in
+	-a*)
+		ALPHABETICAL=1
+		;;
 	-b*)
 		BIMONTHLY=1
 		shift
@@ -601,43 +611,46 @@ function collect_braced_item( count,item,k,s)
     # in the collections after sort_keys().  The list here includes all
     # of the field names defined in the GNU Emacs bibtex.el support
     # code, plus CODEN, day, key, ISBN, ISSN, LCCN, MRnumber, MRclass,
-    # price, and URL.
-
-    item = item get_pair("author")
-    item = item get_pair("editor")
-    item = item get_pair("key")
-    item = item get_pair("booktitle")
-    item = item get_pair("title")
-    item = item get_pair("crossref")
-    item = item get_pair("chapter")
-    item = item get_pair("journal")
-    item = item get_pair("volume")
-    item = item get_pair("type")
-    item = item get_pair("number")
-    item = item get_pair("howpublished")
-    item = item get_pair("institution")
-    item = item get_pair("organization")
-    item = item get_pair("publisher")
-    item = item get_pair("school")
-    item = item get_pair("address")
-    item = item get_pair("edition")
-    item = item get_pair("pages")
-    item = item get_pair("day")
-    item = item get_pair("month")
-    item = item get_pair("year")
-    item = item get_pair("CODEN")
-    item = item get_pair("ISBN")
-    item = item get_pair("ISSN")
-    item = item get_pair("LCCN")
-    item = item get_pair("MRclass")
-    item = item get_pair("MRnumber")
-    item = item get_pair("bibdate")
-    item = item get_pair("bibsource")
-    item = item get_pair("note")
-    item = item get_pair("price")
-    item = item get_pair("series")
-    item = item get_pair("URL")
+    # MRreviewer, price, and URL.
 
+    if (!ALPHABETICAL)
+    {
+	item = item get_pair("author")
+	item = item get_pair("editor")
+	item = item get_pair("key")
+	item = item get_pair("booktitle")
+	item = item get_pair("title")
+	item = item get_pair("crossref")
+	item = item get_pair("chapter")
+	item = item get_pair("journal")
+	item = item get_pair("volume")
+	item = item get_pair("type")
+	item = item get_pair("number")
+	item = item get_pair("howpublished")
+	item = item get_pair("institution")
+	item = item get_pair("organization")
+	item = item get_pair("publisher")
+	item = item get_pair("school")
+	item = item get_pair("address")
+	item = item get_pair("edition")
+	item = item get_pair("pages")
+	item = item get_pair("day")
+	item = item get_pair("month")
+	item = item get_pair("year")
+	item = item get_pair("CODEN")
+	item = item get_pair("ISBN")
+	item = item get_pair("ISSN")
+	item = item get_pair("LCCN")
+	item = item get_pair("MRclass")
+	item = item get_pair("MRnumber")
+	item = item get_pair("MRreviewer")
+	item = item get_pair("bibdate")
+	item = item get_pair("bibsource")
+	item = item get_pair("note")
+	item = item get_pair("price")
+	item = item get_pair("series")
+	item = item get_pair("URL")
+    }
     sort_keys()			# only a few are left at this point
 
     for (k = 1; sorted_keys[k]; ++k) # output them in ordered by key
@@ -688,13 +701,17 @@ function error(msg)
 
 function get_pair(key, s)
 {
-    if (key in key_value_pair)
+    s = ""
+    while (key in key_value_pair)
     {
-	s = key_value_pair[key] "\n"
+	# Handle duplicate keys transparently to caller, so that such
+	# keys always appear consecutively in the ordered output,
+	# independent of the requested sorting order.
+
+	s = s key_value_pair[key] "\n"
 	delete key_value_pair[key]
+	key = key "-z"
     }
-    else
-	s = ""
     return s
 }
 
@@ -821,6 +838,7 @@ function warning(msg)
 # Use GNU gawk instead of nawk: Sun Solaris 2.x nawk often complains
 # `input record too long'.
 gawk \
+	-v ALPHABETICAL=$ALPHABETICAL \
 	-v BIMONTHLY=$BIMONTHLY \
 	-v CHECKMISSING=$CHECKMISSING \
 	-v MONTHLY=$MONTHLY \
